package de.intevation.gnv.geobackend.base.query;

import java.util.Collection;

import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;
import net.sf.ehcache.CacheManager;

import org.apache.log4j.Logger;

import de.intevation.gnv.geobackend.base.Result;

/**
 * @author Sascha L. Teichmann <sascha.teichmann@intevation.de>
 *
 */
public class CachingQueryExecutorFactory
extends      QueryExecutorFactory
{
    public static final String QUERY_EXECUTOR_FACTORY = "caching.query.executor.config";
    public static final String CACHE_NAME             = "sql.cache";

    private static Logger log = Logger.getLogger(CachingQueryExecutorFactory.class);

    protected CacheManager manager;

    public CachingQueryExecutorFactory() {
        log.info("using SQL cache");
        String configFile = System.getProperty(QUERY_EXECUTOR_FACTORY);
        manager = configFile != null
            ? new CacheManager(configFile)
            : new CacheManager();
        manager.addCache(CACHE_NAME);
    }

    public QueryExecutor getQueryExecutor() {
        return new DefaultQueryExceutor() {

            public Collection<Result> cachedResults(String query) {
                Cache   cache   = manager.getCache(CACHE_NAME);
                Element element = cache.get(query);
                if (log.isDebugEnabled()) {
                    log.debug("found results in SQL cache: " + (element != null));
                }
                return element != null
                    ? (Collection<Result>)element.getObjectValue()
                    : null;
            }

            public void cacheResults(String query, Collection<Result> results) {
                log.debug("store results in SQL cache");
                Cache cache = manager.getCache(CACHE_NAME);
                cache.put(new Element(query, results));
            }
        };
    }

    public void shutdown() {
        log.info("shutting down SQL cache");
        manager.getCache(CACHE_NAME).flush();
        manager.shutdown();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
